"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const findNpmPerfix = require("find-npm-prefix");
const command_1 = require("@oclif/command");
const lodash_1 = require("lodash");
const path_1 = require("path");
const packageJson_1 = require("./utilities/packageJson");
const isFile_1 = require("./utilities/isFile");
const isDirectory_1 = require("./utilities/isDirectory");
const realpathAsync_1 = require("./utilities/realpathAsync");
const store_1 = require("./store");
const actions_1 = require("./actions");
class default_1 extends command_1.Command {
    constructor() {
        super(...arguments);
        this.store = store_1.store;
    }
    async init() {
        // tslint:disable-next-line no-any no-shadowed-variable
        const { flags } = this.parse(this.constructor);
        store_1.store.dispatch(actions_1.RESET(undefined));
        store_1.store.dispatch(actions_1.SET_OCLIF_CONFIG(this.config));
        const state = store_1.store.getState();
        const project = await realpathAsync_1.realpathAsync(lodash_1.isUndefined(flags.project) ? process.cwd() : flags.project).catch(() => {
            throw new Error(`The specified path does not exist: '${flags.project}'.`);
        });
        const tests = lodash_1.filter(await Promise.all([
            isFile_1.isFile(project),
            isDirectory_1.isDirectory(project).then(prod => {
                prod.input = path_1.join(prod.input, 'tsconfig.json');
                return prod;
            })
        ]), test => test.test);
        if (tests.length === 0) {
            throw new Error(`The specified path does not exist: '${project}'.`);
        }
        const tsconfig = path_1.resolve(tests[0].input);
        const { content, path } = await packageJson_1.packageJson(path_1.dirname(tsconfig));
        if (state.oclifConfig.root === path_1.dirname(path)) {
            throw new Error('Change the working directory');
        }
        const prefixContext = await findNpmPerfix(path_1.dirname(path));
        const prefixRoot = await findNpmPerfix(state.oclifConfig.root);
        store_1.store.dispatch(actions_1.SET_PREFIX({
            context: path_1.join(prefixContext, 'node_modules'),
            root: path_1.join(prefixRoot, 'node_modules')
        }));
        // process.chdir(dirname(path))
        store_1.store.dispatch(actions_1.SET_CONTEXT(path_1.dirname(path)));
        store_1.store.dispatch(actions_1.SET_PACKAGE_JSON(content));
        store_1.store.dispatch(actions_1.SET_TSCONFIG(tsconfig));
    }
}
default_1.flags = {
    help: command_1.flags.help({ char: 'h' }),
    project: command_1.flags.string({
        char: 'p',
        description: "path to 'tsconfig.json', or to a folder with it",
        required: false
    })
};
exports.default = default_1;
