"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
exports.commandFlags = {
    entry: command_1.flags.string({
        char: 'e',
        description: 'project entry point',
        multiple: true,
        required: false
    }),
    output: command_1.flags.string({
        char: 'o',
        description: '[default: lib] output directory path',
        required: false
    }),
    minimize: command_1.flags.boolean({
        description: '[default: true] minimize javascript',
        allowNo: true
    }),
    stats: command_1.flags.boolean({
        description: 'write JSON file(s) with compilation statistics',
        default: false
    }),
    clean: command_1.flags.boolean({
        description: '[default: true] clean output directory',
        allowNo: true
    }),
    module: command_1.flags.string({
        char: 'm',
        description: 'module code generation (esm is always enabled)',
        multiple: true,
        options: ['cjs', 'umd', 'esm'],
        required: false
    }),
    'machine-readable': command_1.flags.boolean({
        description: 'enables JSON output mode',
        default: false
    })
};
exports.apiExtractFlags = {
    entry: command_1.flags.string({
        char: 'e',
        description: 'd.ts entry point',
        multiple: false,
        required: true
    }),
    output: command_1.flags.string({
        char: 'o',
        description: '[default: api.json] output json file path',
        required: false
    })
};
const TS_COMPILER_OPTIONS = {
    downlevelIteration: true,
    importHelpers: true,
    target: 'es6',
    module: 'esnext',
    moduleResolution: 'node',
    sourceMap: true
};
const WEBPACK_NODE = {
    console: false,
    global: false,
    process: false,
    __filename: false,
    __dirname: false,
    Buffer: false,
    setImmediate: false
};
const LODASH_OPTIONS = {
    cloning: true,
    caching: true,
    collections: true,
    unicode: true,
    memoizing: true,
    coercions: true,
    flattening: true,
    paths: true
};
const MINIFY_OPTIONS = {
    ecma: undefined,
    warnings: false,
    parse: {},
    compress: {
        arrows: true,
        collapse_vars: true,
        comparisons: true,
        computed_props: true,
        hoist_funs: true,
        hoist_props: true,
        hoist_vars: true,
        inline: true,
        loops: true,
        negate_iife: true,
        properties: true,
        reduce_funcs: true,
        reduce_vars: true,
        switches: true,
        toplevel: true,
        typeofs: true,
        // a few flags with noticable gains/speed ratio
        // numbers based on out of the box vendor bundle
        booleans: true,
        if_return: true,
        sequences: true,
        unused: true,
        // required features to drop conditional branches
        conditionals: true,
        dead_code: true,
        evaluate: true
    },
    mangle: true,
    module: false,
    output: null,
    toplevel: false,
    nameCache: null,
    ie8: false,
    keep_classnames: undefined,
    keep_fnames: false,
    safari10: true
};
exports.INITIAL_STATE = {
    options: {},
    runtime: {
        files: {},
        errors: {},
        build: []
    },
    defaults: {
        lodash: {
            id: ['lodash-es', 'lodash', 'lodash-fp'],
            options: LODASH_OPTIONS
        },
        minify: MINIFY_OPTIONS,
        node: WEBPACK_NODE,
        compilerOptions: TS_COMPILER_OPTIONS
    }
};
