"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const api_extractor_1 = require("@microsoft/api-extractor");
const lodash_1 = require("lodash");
const selectors_1 = require("../selectors");
const utilities_1 = require("../utilities");
const logger_1 = require("@escapace/logger");
const path_1 = require("path");
const store_1 = require("../store");
exports.run = async ({ entry, output }) => {
    const state = store_1.store.getState();
    const outputPath = lodash_1.isUndefined(output) ? path_1.resolve(selectors_1.context(state), 'api.json') : path_1.resolve(output);
    if ((await utilities_1.isFile(outputPath)).test) {
        throw new Error(`The file '${outputPath}' already exists`);
    }
    const success = await new Promise(done => {
        // This interface represents the API Extractor config file contents
        const config = {
            apiReviewFile: {
                enabled: false
            },
            apiJsonFile: {
                enabled: true,
                outputFolder: './dist'
            },
            compiler: {
                configType: 'tsconfig',
                rootFolder: selectors_1.context(state)
            },
            project: {
                entryPointSourceFile: path_1.resolve(selectors_1.context(state), entry)
            }
        };
        lodash_1.assign(config, {
            validationRules: {
                missingReleaseTags: 'allow'
            }
        });
        const options = {
            customLogger: {
                logInfo: lodash_1.noop,
                logWarning: lodash_1.noop,
                logVerbose: lodash_1.noop,
                logError: logger_1.logger.error
            },
            localBuild: true,
            typescriptCompilerFolder: path_1.resolve(selectors_1.contextModules(state), 'typescript')
        };
        const extractor = new api_extractor_1.Extractor(config, options);
        if (extractor.processProject()) {
            done(true);
        }
        else {
            done(false);
        }
    });
    const cleanup = () => Promise.resolve()
        .then(() => utilities_1.rimraf(path_1.resolve(selectors_1.context(state), 'dist', 'tsdoc-metadata.json')))
        .then(() => utilities_1.rimraf(path_1.resolve(selectors_1.context(state), 'dist', `${selectors_1.packageName(state)}.api.json`)))
        .then(() => utilities_1.isEmpty(path_1.resolve(selectors_1.context(state), 'dist')))
        .then(t => (t ? utilities_1.rimraf(path_1.resolve(selectors_1.context(state), 'dist')) : true));
    if (success) {
        const data = await utilities_1.readFileAsync(path_1.resolve(selectors_1.context(state), 'dist', `${selectors_1.packageName(state)}.api.json`)).then(
        // tslint:disable-next-line no-console
        d => d.toString());
        await cleanup();
        await utilities_1.writeFileAsync(outputPath, data);
    }
    else {
        await cleanup();
    }
};
