"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const babel = require("gulp-babel");
const gulp = require("gulp");
const gulpFilter = require("gulp-filter");
const gulpTap = require("gulp-tap");
const merge = require("merge2");
const path_1 = tslib_1.__importDefault(require("path"));
const resolveFrom = require("resolve-from");
const sourcemaps = require("gulp-sourcemaps");
const typescript = require("gulp-typescript");
const actions_1 = require("../actions");
const lodash_1 = require("lodash");
const errors_1 = require("./errors");
const store_1 = require("../store");
const selectors_1 = require("../selectors");
exports.gulpBuild = async () => {
    const state = store_1.store.getState();
    const compiler = await Promise.resolve().then(() => tslib_1.__importStar(require(resolveFrom(selectors_1.contextModules(state), 'typescript'))));
    return new Promise(resolve => {
        const result = {
            module: 'esm',
            assets: [],
            errors: [],
            hasErrors: false,
            stats: undefined
        };
        const project = typescript.createProject(selectors_1.tsconfig(state), Object.assign({ noEmitOnError: true, typescript: compiler, sourceRoot: undefined }, selectors_1.compilerOptions(state), { removeComments: false, outDir: selectors_1.context(state), watch: false, sourceMap: true, inlineSourceMap: true, inlineSources: true }));
        const stream = project
            .src()
            .pipe(sourcemaps.init())
            .pipe(project({
            error(error) {
                const rendered = errors_1.normalizeGulpError(error, compiler);
                if (!lodash_1.includes(result.errors, rendered)) {
                    result.errors.push(rendered);
                }
            },
            finish(data) {
                const showErrorCount = (count) => {
                    if (count === 0)
                        return;
                    result.hasErrors = true;
                };
                store_1.store.dispatch(actions_1.SET_ROOTDIR(lodash_1.isUndefined(project.options.rootDir) ? selectors_1.context(state) : project.options.rootDir));
                showErrorCount(data.transpileErrors);
                showErrorCount(data.optionsErrors);
                showErrorCount(data.syntaxErrors);
                showErrorCount(data.globalErrors);
                showErrorCount(data.semanticErrors);
                showErrorCount(data.declarationErrors);
                showErrorCount(data.emitErrors);
            }
        }))
            .on('error', lodash_1.noop);
        const specFilter = gulpFilter(file => !/\.spec\.js$/.test(file.path));
        const specTypesFilter = gulpFilter(file => !/\.spec\.d\.ts$/.test(file.path));
        // const sourcemapFilter = gulpFilter(file => !/\.map$/.test(file.path))
        const ds = selectors_1.declaration(state)
            ? stream.dts.pipe(specTypesFilter).pipe(gulp.dest(selectors_1.outputPathTypes(state)))
            : undefined;
        const es = stream.js
            .pipe(specFilter)
            // tslint:disable-next-line no-any
            .pipe(babel(selectors_1.gulpBabelOptions(state)))
            .pipe(sourcemaps.write('.', {
            includeContent: true
        }))
            .pipe(gulp.dest(selectors_1.outputPathEsm(state)))
            .pipe(gulpTap(file => {
            if (path_1.default.extname(file.path) === '.js') {
                result.assets.push(file.path);
            }
        }));
        merge(lodash_1.compact([ds, es]))
            .on('finish', () => {
            resolve(result);
        })
            .on('error', error => {
            if (error && error.message) {
                result.errors.push(error.message);
            }
            result.hasErrors = true;
            resolve(result);
        });
    }).then(result => store_1.store.dispatch(actions_1.BUILD_RESULT(result)));
};
