"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// import path from 'path'
const gzipSize = require("gzip-size");
const prettyBytes = require("pretty-bytes");
const selectors_1 = require("../selectors");
const utilities_1 = require("../utilities");
const errors_1 = require("./errors");
const logger_1 = require("@escapace/logger");
const store_1 = require("../store");
const lodash_1 = require("lodash");
exports.report = async () => {
    const results = selectors_1.buildResults(store_1.store.getState());
    const fail = selectors_1.buildResultsWithErrors(store_1.store.getState());
    if (!lodash_1.isEmpty(fail)) {
        await errors_1.dispatchFilesFromErrors();
        errors_1.reportErrors();
        lodash_1.forEach(lodash_1.uniq(lodash_1.flatten(lodash_1.map(fail, f => f.errors))), s => logger_1.logger.error(s));
        throw new Error('Recce could not finish the build');
    }
    const reports = lodash_1.fromPairs(await Promise.all(lodash_1.map(results, async (result) => {
        let size = 0;
        let buf = Buffer.alloc(0);
        for (const p of result.assets) {
            const asset = await utilities_1.readFileAsync(p);
            const tmpBuf = Buffer.concat([asset, buf]);
            buf = tmpBuf;
            size += asset.length;
        }
        const gSize = await gzipSize(buf);
        return [
            result.module,
            {
                assets: result.assets,
                gzipSize: gSize,
                size
            }
        ];
    })));
    if (selectors_1.machineReadable(store_1.store.getState())) {
        // tslint:disable-next-line no-console
        console.log(JSON.stringify(reports, null, '  '));
    }
    else {
        logger_1.logger.log('');
        // tslint:disable-next-line no-shadowed-variable
        lodash_1.forEach(reports, ({ gzipSize, size }, key) => {
            logger_1.logger.log(`${lodash_1.toUpper(key)}: ${prettyBytes(size)} (${prettyBytes(gzipSize)} gzipped)`);
        });
    }
};
