"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const webpack = require("webpack");
const lodash_1 = require("lodash");
const selectors_1 = require("../selectors");
const store_1 = require("../store");
const actions_1 = require("../actions");
exports.webpackBuild = async (module) => {
    const callback = (result) => store_1.store.dispatch(actions_1.BUILD_RESULT(result));
    const configuration = selectors_1.webpackConfiguration(module)(store_1.store.getState());
    const compiler = webpack(configuration);
    const method = (handler) => {
        const state = store_1.store.getState();
        if (selectors_1.condBuild(state)) {
            return compiler.run(handler);
        }
        else if (selectors_1.condWatch(state)) {
            return compiler.watch({
                aggregateTimeout: 300,
                poll: true
            }, handler);
        }
    };
    return new Promise(resolve => {
        const watching = method(
        // tslint:disable-next-line no-any
        (err, stats) => {
            const result = {
                module,
                assets: [],
                errors: [],
                hasErrors: false,
                stats: undefined
            };
            const info = stats.toJson({
                all: true
            });
            result.stats = info;
            if (!lodash_1.isNull(err) || stats.hasErrors() === true) {
                result.hasErrors = true;
                result.assets = lodash_1.map(info.assets, asset => asset.name);
                if (lodash_1.isNull(err)) {
                    result.errors = lodash_1.map(stats.compilation.errors, value => value.toString());
                }
                else {
                    result.errors = lodash_1.concat(lodash_1.compact([err.message, err.details]));
                }
                resolve({
                    close: lodash_1.isUndefined(watching) ? lodash_1.noop : watching.close,
                    compiler: compiler,
                    invalidate: lodash_1.isUndefined(watching) ? lodash_1.noop : watching.invalidate
                });
                callback(result);
            }
            else {
                result.assets = lodash_1.filter(lodash_1.map(info.assets, asset => path_1.default.join(info.outputPath, asset.name)), p => !/\.js\.map/.test(p));
                resolve({
                    close: lodash_1.isUndefined(watching) ? lodash_1.noop : watching.close,
                    compiler: compiler,
                    invalidate: lodash_1.isUndefined(watching) ? lodash_1.noop : watching.invalidate
                });
                callback(result);
            }
        });
    });
};
