"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mm = require("micromatch");
const lodash_1 = require("lodash");
class FilterWebpackPlugin {
    constructor(options) {
        this.options = lodash_1.defaults({}, options, { select: false, patterns: [] });
    }
    apply(compiler) {
        const filter = this.options.select === true ? lodash_1.pick : lodash_1.omit;
        compiler.hooks.emit.tapAsync('RecceFilterWebpackPlugin', (compilation, callback) => {
            if (this.options.patterns.length > 0) {
                const files = lodash_1.keys(compilation.assets);
                const matchedFiles = mm(files, this.options.patterns, { basename: true });
                compilation.assets = filter(compilation.assets, matchedFiles);
            }
            callback();
        });
    }
}
exports.FilterWebpackPlugin = FilterWebpackPlugin;
