"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const reselect_1 = require("reselect");
const resolveFrom = require("resolve-from");
const semver = require("semver");
const lodash_1 = require("lodash");
const lodash_2 = require("./lodash");
const ramda_1 = require("./ramda");
const general_1 = require("./general");
const nodeTarget = reselect_1.createSelector(general_1.packageJson, (pjson) => {
    const node = lodash_1.get(pjson, 'engines.node');
    if (lodash_1.isString(node)) {
        if (semver.valid(node) !== null) {
            return node;
        }
        const coerced = semver.coerce(node);
        // tslint:disable-next-line strict-type-predicates
        if (semver.validRange(node) !== null && coerced !== null) {
            return coerced.version;
        }
    }
    return `${semver.major(process.versions.node)}.0.0`;
});
exports.babelOptions = (module) => (state) => ({
    cacheDirectory: false,
    babelrc: false,
    plugins: lodash_1.compact([
        resolveFrom(general_1.rootModules(state), '@babel/plugin-syntax-dynamic-import'),
        module === 'esm'
            ? resolveFrom(general_1.rootModules(state), 'babel-plugin-annotate-pure-calls')
            : undefined,
        module === 'esm' && lodash_2.condLodash(state)
            ? [
                resolveFrom(general_1.rootModules(state), 'babel-plugin-lodash'),
                {
                    id: lodash_2.lodashId(state),
                    cwd: general_1.context(state)
                }
            ]
            : undefined,
        module === 'esm' && ramda_1.condRamda(state)
            ? [
                resolveFrom(general_1.rootModules(state), 'babel-plugin-ramda'),
                {
                    useES: true
                }
            ]
            : undefined
    ]),
    presets: lodash_1.compact([
        module !== 'esm'
            ? [
                resolveFrom(general_1.rootModules(state), '@babel/preset-env'),
                {
                    configPath: general_1.context(state),
                    exclude: ['transform-async-to-generator', 'transform-regenerator'],
                    useBuiltIns: false,
                    modules: false,
                    loose: true,
                    ignoreBrowserslistConfig: module === 'cjs',
                    targets: module === 'cjs'
                        ? {
                            node: nodeTarget(state)
                        }
                        : undefined
                }
            ]
            : undefined
    ])
});
exports.gulpBabelOptions = (state) => lodash_1.omit(exports.babelOptions('esm')(state), ['cacheDirectory']);
