"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const reselect_1 = require("reselect");
const path_1 = require("path");
const lodash_1 = require("lodash");
exports.buildResults = (state) => state.runtime.build;
exports.compilerOptions = (state) => state.options.compilerOptions;
exports.condClean = (state) => state.options.clean;
exports.condMinimize = (state) => state.options.minimize;
exports.context = (state) => state.options.context;
exports.contextModules = (state) => state.options.prefix.context;
exports.declaration = (state) => !!state.options.compilerOptions.declaration;
exports.files = (state) => state.runtime.files;
exports.machineReadable = (state) => state.options.machineReadable;
exports.mode = (state) => state.options.mode;
exports.modules = (state) => state.options.modules;
exports.nodeOptions = (state) => state.defaults.node;
exports.packageJson = (state) => state.options.pjson;
exports.rootDir = (state) => state.options.rootDir;
exports.rootModules = (state) => state.options.prefix.root;
exports.tsErrors = (state) => state.runtime.errors;
exports.tsconfig = (state) => state.options.tsconfig;
exports.buildResultsWithErrors = reselect_1.createSelector(exports.buildResults, results => lodash_1.filter(results, result => result.hasErrors));
exports.condBuildWithErrors = reselect_1.createSelector(exports.buildResultsWithErrors, a => a.length !== 0);
const _entries = (state) => state.options.entries;
exports.entries = reselect_1.createSelector(_entries, exports.context, (ents, ctx) => lodash_1.map(ents, ent => path_1.resolve(ctx, ent)));
const _outputPath = (state) => state.options.outputPath;
exports.outputPath = reselect_1.createSelector(exports.context, _outputPath, path_1.resolve);
exports.outputPathEsm = reselect_1.createSelector(exports.outputPath, o => path_1.join(o, 'esm'));
exports.outputPathCjs = reselect_1.createSelector(exports.outputPath, o => path_1.join(o, 'cjs'));
exports.outputPathUmd = reselect_1.createSelector(exports.outputPath, o => path_1.join(o, 'umd'));
exports.outputPathTypes = reselect_1.createSelector(exports.outputPath, o => path_1.join(o, 'types'));
exports.condWatch = reselect_1.createSelector(exports.mode, () => false);
exports.condBuild = reselect_1.createSelector(exports.mode, m => m === 'build');
exports.dependencies = reselect_1.createSelector(exports.packageJson, pj => pj.dependencies);
exports.devDependencies = reselect_1.createSelector(exports.packageJson, pj => pj.devDependencies);
exports.combinedDependencies = reselect_1.createSelector(exports.devDependencies, exports.dependencies, (dev, dep) => lodash_1.assign({}, dev, dep));
exports.packageName = reselect_1.createSelector(exports.packageJson, pj => pj.name);
