"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const TerserPlugin = require("terser-webpack-plugin");
const lodashPlugin = require("lodash-webpack-plugin");
const nodeExternals = require("webpack-node-externals");
const resolveFrom = require("resolve-from");
const plugins_1 = require("../plugins");
const reselect_1 = require("reselect");
const path_1 = require("path");
const lodash_1 = require("lodash");
const tsconfig_paths_webpack_plugin_1 = tslib_1.__importDefault(require("tsconfig-paths-webpack-plugin"));
const general_1 = require("./general");
const lodash_2 = require("./lodash");
const babel_1 = require("./babel");
const minifyOptions = (state) => state.defaults.minify;
const webpackEntries = reselect_1.createSelector(general_1.entries, general_1.rootDir, general_1.outputPathEsm, general_1.context, (ents, root, ope, ctxx) => lodash_1.fromPairs(lodash_1.map(ents, ent => [
    path_1.parse(ent).name,
    `./${path_1.relative(ctxx, path_1.resolve(ope, path_1.relative(root, path_1.resolve(root, ent)).replace(/\.ts$/, '.js')))}`
])));
const webpackRules = (module) => (state) => [
    {
        test: /\.js$/,
        use: resolveFrom(general_1.rootModules(state), 'source-map-loader'),
        enforce: 'pre'
    },
    {
        test: /\.js$/,
        exclude: /tslib/,
        use: [
            {
                loader: resolveFrom(general_1.rootModules(state), 'babel-loader'),
                options: babel_1.babelOptions(module)(state)
            }
        ]
    }
];
exports.webpackConfiguration = (module) => (state) => ({
    name: module,
    cache: false,
    context: general_1.context(state),
    target: module === 'cjs' ? 'node' : 'web',
    externals: module === 'cjs'
        ? [
            nodeExternals({
                whitelist: ['lodash-es', /^ramda\/es/]
            })
        ]
        : undefined,
    mode: 'production',
    entry: webpackEntries(state),
    devtool: 'source-map',
    output: {
        libraryTarget: module === 'cjs' ? 'commonjs2' : 'umd',
        library: module === 'cjs' ? undefined : lodash_1.camelCase(general_1.packageName(state)),
        path: module === 'cjs' ? general_1.outputPathCjs(state) : general_1.outputPathUmd(state),
        filename: `[name].js`
    },
    module: {
        rules: webpackRules(module)(state)
    },
    optimization: {
        nodeEnv: false,
        minimize: false
    },
    plugins: lodash_1.compact([
        // new DuplicatePackageCheckerPlugin({
        //   verbose: true,
        //   strict: true,
        //   showHelp: false,
        //   emitError: true
        // }),
        general_1.condWatch(state) ? new plugins_1.DoneHookWebpackPlugin() : undefined,
        // new webpack.NoEmitOnErrorsPlugin(),
        lodash_2.condLodash(state) ? new lodashPlugin(lodash_2.lodashOptions(state)) : undefined,
        general_1.condMinimize(state)
            ? new TerserPlugin({
                terserOptions: minifyOptions(state),
                sourceMap: true,
                cache: false,
                parallel: true
            })
            : undefined,
        new plugins_1.FilterWebpackPlugin({
            patterns: ['*.d.ts']
        })
    ]),
    resolve: {
        plugins: [
            new tsconfig_paths_webpack_plugin_1.default({
                configFile: general_1.tsconfig(state),
                mainFields: module === 'umd' ? ['module', 'browser', 'main'] : ['module', 'main'],
                silent: true
            })
        ],
        symlinks: true,
        extensions: ['.ts', '.js', '.tsx', '.json'],
        modules: [general_1.contextModules(state)],
        mainFields: module === 'umd' ? ['module', 'browser', 'main'] : ['module', 'main']
    },
    resolveLoader: {
        modules: [general_1.rootModules(state)]
    },
    node: module === 'cjs' ? general_1.nodeOptions(state) : undefined
});
