"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const immer_1 = tslib_1.__importDefault(require("immer"));
const redux_1 = require("redux");
const lodash_1 = require("lodash");
const constants_1 = require("../constants");
const actions_1 = require("../actions");
function isType(action, actionCreator) {
    return action.type === actionCreator.type;
}
exports.isType = isType;
const reducer = (state = constants_1.INITIAL_STATE, action) => {
    return immer_1.default(state, draft => {
        if (isType(action, actions_1.SET_OCLIF_CONFIG)) {
            draft.oclifConfig = action.payload;
        }
        else if (isType(action, actions_1.SET_CONTEXT)) {
            draft.options.context = action.payload;
        }
        else if (isType(action, actions_1.SET_PACKAGE_JSON)) {
            draft.options.pjson = action.payload;
        }
        else if (isType(action, actions_1.SET_MODE)) {
            draft.options.mode = action.payload;
        }
        else if (isType(action, actions_1.SET_BUILD_OPTIONS)) {
            lodash_1.defaults(draft.options, action.payload);
        }
        else if (isType(action, actions_1.SET_PREFIX)) {
            draft.options.prefix = action.payload;
        }
        else if (isType(action, actions_1.SET_ROOTDIR)) {
            draft.options.rootDir = action.payload;
        }
        else if (isType(action, actions_1.ADD_TYPESCRIPT_ERROR)) {
            const errors = draft.runtime.errors;
            if (lodash_1.isUndefined(errors[action.payload.hash])) {
                errors[action.payload.hash] = action.payload;
            }
            else {
                errors[action.payload.hash].modules = lodash_1.union(errors[action.payload.hash].modules, action.payload.modules);
            }
        }
        else if (isType(action, actions_1.ADD_FILE_SOURCE)) {
            draft.runtime.files[action.payload.file] = action.payload.source;
        }
        else if (isType(action, actions_1.RESET_FILE_SOURCES)) {
            draft.runtime.files = {};
        }
        else if (isType(action, actions_1.SET_TSCONFIG)) {
            draft.options.tsconfig = action.payload;
        }
        else if (isType(action, actions_1.BUILD_RESULT)) {
            draft.runtime.build.push(action.payload);
        }
        else if (isType(action, actions_1.RESET)) {
            lodash_1.assign(draft, constants_1.INITIAL_STATE);
        }
        else if (isType(action, actions_1.RESET_TYPESCRIPT_ERRORS)) {
            draft.runtime.errors = {};
        }
    });
};
exports.store = redux_1.createStore(reducer);
